import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

///InputWidgetDel
typedef void InputWidgetDelCallBack(String content);

enum ITextInputType {
  text,
  multiline,
  number,
  phone,
  datetime,
  emailAddress,
  url,
  password
}

class EditTextWidget extends StatefulWidget {
  final ITextInputType keyboardType;
  final int? minLines;
  final int maxLines;
  final int? inputMaxLength;
  final String inputText;
  final String? hintText;
  final TextStyle? hintStyle;
  final InputWidgetDelCallBack? fieldCallBack;
  final Icon? deleteIcon;
  final InputBorder? inputBorder;
  final Color? fillColor;
  final Widget? prefixIcon;
  final TextStyle? textStyle;
  final TextAlign textAlign;
  final FormFieldValidator<String>? validator;
  final bool enabled;
  final ValueChanged<String>? onSubmitted;
  final FocusNode? focusNode;
  final Color cursorColor;

  EditTextWidget({
    Key? key,
    ITextInputType keyboardType: ITextInputType.text,
    this.minLines,
    this.maxLines = 1,
    this.hintText,
    this.hintStyle,
    this.fieldCallBack,
    this.deleteIcon,
    this.inputBorder,
    this.textStyle,
    this.prefixIcon,
    this.validator,
    this.fillColor,
    this.inputText = '',
    this.textAlign = TextAlign.start,
    this.enabled = true,
    this.inputMaxLength,
    this.onSubmitted,
    this.focusNode,
    required this.cursorColor,
  })  : assert(maxLines > 0),
        keyboardType = maxLines == 1 ? keyboardType : ITextInputType.multiline,
        super(key: key);

  @override
  State<EditTextWidget> createState() {
    return _EditTextWidgetState();
  }
}

class _EditTextWidgetState extends State<EditTextWidget> {
  String _inputText = '';
  bool _isNumber = false;
  bool _isPassword = false;
  TextEditingController _textEditingController = TextEditingController();

  ///输入类型
  // ignore: missing_return
  TextInputType _getTextInputType() {
    switch (widget.keyboardType) {
      case ITextInputType.text:
        return TextInputType.text;
      case ITextInputType.multiline:
        return TextInputType.multiline;
      case ITextInputType.number:
        _isNumber = true;
        return TextInputType.number;
      case ITextInputType.phone:
        _isNumber = true;
        return TextInputType.phone;
      case ITextInputType.datetime:
        return TextInputType.datetime;
      case ITextInputType.emailAddress:
        return TextInputType.emailAddress;
      case ITextInputType.url:
        return TextInputType.url;
      case ITextInputType.password:
        _isPassword = true;
        return TextInputType.text;
    }
  }

  ///输入范围
  List<TextInputFormatter> _getTextInputFormatter() {
    return _isNumber
        ? <TextInputFormatter>[
            FilteringTextInputFormatter.digitsOnly,
            LengthLimitingTextInputFormatter(widget.inputMaxLength)
          ]
        : <TextInputFormatter>[
            LengthLimitingTextInputFormatter(widget.inputMaxLength)
          ];
  }

  @override
  void initState() {
    super.initState();
    _inputText = widget.inputText;
    _textEditingController.text = _inputText;
  }

  ///外部文字更新之后更新内部文字
  @override
  void didUpdateWidget(EditTextWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.inputText.isEmpty == false) {
      _inputText = widget.inputText;
      _textEditingController.text = _inputText;
      _textEditingController.selection = TextSelection.fromPosition(
          TextPosition(
              affinity: TextAffinity.downstream, offset: _inputText.length));
    }
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
        cursorColor: widget.cursorColor,
        controller: _textEditingController,
        decoration: InputDecoration(
            hintStyle: widget.hintStyle,
            hintText: widget.hintText,
            enabledBorder: widget.inputBorder ?? UnderlineInputBorder(),
            focusedBorder: widget.inputBorder ?? UnderlineInputBorder(),
            fillColor: widget.fillColor ?? Colors.transparent,
            filled: true,
            prefixIcon: widget.prefixIcon),
        onChanged: (str) {
          if (!mounted) return;
          setState(() {
            _inputText = str;
            widget.fieldCallBack?.call(_inputText);
          });
        },
        focusNode: widget.focusNode ?? FocusNode(),
        textAlign: widget.textAlign,
        keyboardType: _getTextInputType(),
        minLines: widget.minLines,
        maxLines: widget.maxLines,
        inputFormatters: _getTextInputFormatter(),
        style: widget.textStyle,
        obscureText: _isPassword,
        enabled: widget.enabled,
        onSubmitted: widget.onSubmitted);
  }
}
